'use strict';Object.defineProperty(exports, "__esModule", { value: true });exports.API = exports.Procedure = exports.Neo4jConnection = undefined;



var _neo4jDriver = require('neo4j-driver');
var _fs = require('fs');var _fs2 = _interopRequireDefault(_fs);
var _path = require('path');var _path2 = _interopRequireDefault(_path);
var _chai = require('chai');var _chai2 = _interopRequireDefault(_chai);
var _chalk = require('chalk');var _chalk2 = _interopRequireDefault(_chalk);
var _parseNeo4j = require('parse-neo4j');
var _procedure = require('./procedure');function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}

class Neo4jConnection {
    constructor({ boltUrl, user, password } = {}) {
        this.queries = {};

        this.driver = _neo4jDriver.v1.driver(boltUrl, _neo4jDriver.v1.auth.basic(user, password));
        const session = this.driver.session();
        this.initialized = session.run('RETURN "Neo4j instance successfully connected."').
        then(result => {
            console.log(_chalk2.default.green((0, _parseNeo4j.parse)(result)));
            session.close();
        }).
        catch(error => {
            console.error(
            _chalk2.default.red('Error connecting to the Neo4j instance, check connection options'));
            console.log(error);
            throw error;
        });
    }

    addCypherQueryFile(cypherQueryFilePath) {
        this.queries[cypherQueryFilePath] = _fs2.default.readFileSync(cypherQueryFilePath, 'utf8');
    }

    async executeCypher(cypherQueryOrQueryFilePath, queryParams, pathIsQuery = false) {
        const _executeCypher = (query, queryParams) => {
            return new Promise((resolve, reject) => {
                const session = this.driver.session();

                session.run(query, queryParams).
                then(result => {
                    session.close();
                    resolve(result);
                }).
                catch(error => {
                    session.close();
                    error = error.fields ? JSON.stringify(error.fields[0]) : String(error);
                    reject(`error while executing Cypher: ${error}`);
                });
            }).then(_parseNeo4j.parse);
        };

        if (!pathIsQuery) {
            cypherQueryOrQueryFilePath = _path2.default.resolve(process.cwd(), cypherQueryOrQueryFilePath);
            if (!this.queries[cypherQueryOrQueryFilePath])
            this.addCypherQueryFile(cypherQueryOrQueryFilePath);
        }

        const query = pathIsQuery ? cypherQueryOrQueryFilePath :
        this.queries[cypherQueryOrQueryFilePath];

        let result = [];

        if (Array.isArray(query) && query.length)
        for (const entry of query)
        result.push((await _executeCypher(entry, queryParams)));else


        result = await _executeCypher(query, queryParams);
        return result;
    }} /**
        * Created by keyvan on 8/16/16.
        */
class API {
    constructor(neo4jConnection, options) {
        if (typeof options.procedure === 'function')
        this.invoke = options.procedure;else

        this.invoke = (0, _procedure.createProcedure)(neo4jConnection, options);

        _chai2.default.assert.typeOf(options.method, 'string');
        _chai2.default.assert.typeOf(options.route, 'string');

        this.method = options.method;
        this.route = options.route;
        this.allowedRoles = options.allowedRoles || [];
        _chai2.default.assert.isArray(this.allowedRoles);
        this.allowedRoles = this.allowedRoles.map(role => role.toLowerCase());
        this.requiresJwtAuthentication = this.allowedRoles.length > 0;
    }}exports.


Neo4jConnection = Neo4jConnection;exports.Procedure = _procedure.Procedure;exports.API = API;